/*****************
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Clase base para el procesamiento del xml del modelo de datos.
- ENGLISH -
Base class for data model xml processing.
*****************/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING PROGRESS.Lang.AppError.
USING com.nomadesoft.lib.*.
USING com.nomadesoft.ABLaUML.repository.*.
USING com.nomadesoft.ABLaUML.ABLXMLWorker.

/**
Se encarga de crear todos los objetos y paquetes necesarios para luego poder procesar las relaciones
entre ellos.
**/
CLASS com.nomadesoft.ABLaUML.DDWorker INHERITS ABLXMLWorker:
  DEFINE PUBLIC PROPERTY cchTableTagsMap AS CHARACTER INITIAL 'cancreate,TableCanCreate,candelete,TableCanDelete,candump,TableCanDump,canload,TableCanLoad,canread,TableCanRead,canwrite,TableCanWrite,creator,TableCreator,dblang,TableOrigin,defpk,DefaultPrimaryKey,dumpname,TableDumpName,label,TableLabel,salabel,TableLabelStringAttr,frozen,Frozen,hidden,Hidden,owner,Owner,valexp,DeleteValExp,valmsg,DeleteValMsg,valmsgsa,DeleteValMsgStringAttr' NO-UNDO
    GET.
    PROTECTED SET.

  DEFINE PUBLIC PROPERTY cchColumnTagsMap AS CHARACTER INITIAL 'canread,ColumnCanRead,canwrite,ColumnCanWrite,charset,CharacterSet,collabel,ColumnLabel,collabelsa,ColumnLabelStringAttr,collation,Collation,decimals,Decimals,extent,Extent,position,Position,sensitive,CaseSensitive,format,Format,formatsa,DefaultFormatStringAttr,help,Help,helpsa,HelpStringAttr,initialsa,InitialStringAttr,label,SideLabel,labelsa,SideLabelStringAttr,order,Order,sysfield,NoDefaultDisplay,valexp,ValExp,valmsg,ValMsg,valmsgsa,ValMsgStringAttr,viewas,ViewAsPhrase,width,Width' NO-UNDO
    GET.
    PROTECTED SET.

  DEFINE PUBLIC PROPERTY cchSequenceTagsMap AS CHARACTER INITIAL 'initial,SeqInitial,number,SeqNumber,increment,SeqIncrement,min,SeqMin,max,SeqMax,cycle,SeqCycleOnLimit,owner,Owner' NO-UNDO
    GET.
    PROTECTED SET.

  CONSTRUCTOR DDWorker(INPUT ipobUtils AS CLASS IUtils):
    SUPER(ipobUtils).
  END CONSTRUCTOR.

  /* crea todos los tags necesarios en funcin del mapeo y los atributos del elemento actual.
  Segn el primer parmetro lo hace para:
   1- Objeto
   2- Atributo
   3- Operacin
   4- Connector */
  METHOD PROTECTED VOID CreateTags(INPUT ipinQue AS INTEGER, INPUT ipinID AS INTEGER, INPUT ipchTagsMap AS CHARACTER):
    DEFINE VARIABLE minIndex AS INTEGER     NO-UNDO.
    DEFINE VARIABLE mchValue AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchEntry AS CHARACTER   NO-UNDO.
  
    cobParser:DoLog(STRING(THIS-OBJECT), SUBSTITUTE("Creando Tags &1",ipchTagsMap),"DEBUG").
    IF ipinID EQ ? THEN
      RETURN.
    DO minIndex = 1 TO NUM-ENTRIES(ipchTagsMap) BY 2:
      mchValue = cobParser:AttributeValue(ENTRY(minIndex,ipchTagsMap)).
      IF mchValue EQ ? THEN
        NEXT.
      mchEntry = ENTRY(minIndex + 1,ipchTagsMap).
      CASE ipinQue:
        WHEN 1 THEN DO:
          cobParser:DoLog(STRING(THIS-OBJECT), SUBSTITUTE("ObjectTag &1=&2",mchEntry,mchValue),"DEBUG").
          cobUtils:SetObjectTag (
            input  ipinID, 
            input  mchEntry,
            input  mchValue
            ).
        END.
        WHEN 2 THEN DO:
          cobParser:DoLog(STRING(THIS-OBJECT), SUBSTITUTE("AttributeTag &1=&2",mchEntry,mchValue),"DEBUG").
          cobUtils:SetAttributeTag (
            input  ipinID, 
            input  mchEntry,
            input  mchValue
            ).
        END.
        WHEN 3 THEN DO:
          cobParser:DoLog(STRING(THIS-OBJECT), SUBSTITUTE("OperationTag &1=&2",mchEntry,mchValue),"DEBUG").
          cobUtils:SetOperationTag (
            input  ipinID, 
            input  mchEntry,
            input  mchValue
            ).
        END.
        WHEN 4 THEN DO:
          cobParser:DoLog(STRING(THIS-OBJECT), SUBSTITUTE("ConnectorTag &1=&2",mchEntry,mchValue),"DEBUG").
          cobUtils:SetConnectorTag (
            input  ipinID, 
            input  mchEntry,
            input  mchValue
            ).
        END.
      END CASE.
    END.
  END METHOD.

  METHOD PROTECTED VOID CreateObjectTags(INPUT ipinObjectID AS INTEGER, INPUT ipchTagsMap AS CHARACTER):
    CreateTags(1,ipinObjectID,ipchTagsMap).
  END METHOD.
  METHOD PROTECTED VOID CreateAttributeTags(INPUT ipinAttributeID AS INTEGER, INPUT ipchTagsMap AS CHARACTER):
    CreateTags(2,ipinAttributeID,ipchTagsMap).
  END METHOD.
  METHOD PROTECTED VOID CreateOperationTags(INPUT ipinOperationID AS INTEGER, INPUT ipchTagsMap AS CHARACTER):
    CreateTags(3,ipinOperationID,ipchTagsMap).
  END METHOD.
  METHOD PROTECTED VOID CreateConnectorTags(INPUT ipinConnectorID AS INTEGER, INPUT ipchTagsMap AS CHARACTER):
    CreateTags(4,ipinConnectorID,ipchTagsMap).
  END METHOD.

  /* mtodos que los hijos sobrescribirn para hacer el trabajo real, invocados desde la implementacin de IXMLWorker */
  METHOD PROTECTED VOID DatabaseStart():
  END METHOD.
  METHOD PROTECTED VOID TableStart():
  END METHOD.
  METHOD PROTECTED VOID FieldStart():
  END METHOD.
  METHOD PROTECTED VOID IndexStart():
  END METHOD.
  METHOD PROTECTED VOID StorageAreaStart():
  END METHOD.
  METHOD PROTECTED VOID SequenceStart():
  END METHOD.
  METHOD PROTECTED VOID UnknownStart():
    cobParser:doLog(STRING(THIS-OBJECT),'Comienzo elemento desconocido','WARNING').
  END METHOD.
  METHOD PROTECTED VOID DatabaseEnd():
  END METHOD.
  METHOD PROTECTED VOID TableEnd():
  END METHOD.
  METHOD PROTECTED VOID IndexEnd():
  END METHOD.
  METHOD PROTECTED VOID FieldEnd():
  END METHOD.
  METHOD PROTECTED VOID StorageAreaEnd():
  END METHOD.
  METHOD PROTECTED VOID SequenceEnd():
  END METHOD.
  METHOD PROTECTED VOID UnknownEnd():
    cobParser:doLog(STRING(THIS-OBJECT),'Fin elemento desconocido','WARNING').
  END METHOD.

  /* implementacin de IXMLWorker */
  METHOD PUBLIC OVERRIDE VOID ElementStart(INPUT ipobParser AS CLASS IXMLParser):
    SUPER:ElementStart(ipobParser).
    case cobParser:CurrentElement():
      when "database" THEN
        DatabaseStart().
      WHEN 'table' THEN
        TableStart().  
      WHEN 'storagearea' THEN
        StorageAreaStart().
      WHEN 'field' THEN
        FieldStart().
      WHEN 'index' THEN
        IndexStart().
      WHEN 'sequence' THEN
        SequenceStart().
      OTHERWISE
          UnknownStart().
    END.
  END METHOD.
  METHOD PUBLIC OVERRIDE VOID ElementEnd(INPUT ipobParser AS CLASS IXMLParser):
    SUPER:ElementEnd(ipobParser).
    case cobParser:CurrentElement():
      when "database" THEN
        DatabaseEnd().
      WHEN 'table' THEN
        TableEnd().  
      WHEN 'storagearea' THEN
        StorageAreaEnd().
      WHEN 'field' THEN
        FieldEnd().
      WHEN 'index' THEN
        IndexEnd().
      WHEN 'sequence' THEN
        SequenceEnd().
      OTHERWISE
          UnknownEnd().
    END.
  END METHOD.
END CLASS.
