/*****************
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Crea los diagramas para los paquetes correspondientes a unidades
funcionales y agrega todos los elementos que forman parte del
diagrama, es decir todo lo que est relacionado a la unidad
funcional y sus componentes.
- ENGLISH -
Creates functional unit diagrams, adds all the elements that
belongs to the diagram, that is all the package components and
the related components/packages.
*****************/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING PROGRESS.Lang.AppError.
USING com.nomadesoft.ABLaUML.FUWorker.
USING com.nomadesoft.lib.*.
USING com.nomadesoft.ABLaUML.repository.*.

CLASS com.nomadesoft.ABLaUML.DiagramWorker INHERITS FUWorker:
  DEFINE PUBLIC PROPERTY clgMainDiagram AS LOGICAL INITIAL TRUE NO-UNDO
    GET.
    SET.

  DEFINE PUBLIC PROPERTY cchMainDiagramName AS CHARACTER INITIAL 'Vista de Componentes' NO-UNDO
    GET.
    SET.

  DEFINE PRIVATE VARIABLE cchFUName AS CHARACTER NO-UNDO.

  /* se utiliza cinCurrentObjectID para contener el id del diagrama actual */
  METHOD PROTECTED OVERRIDE VOID UnknownStart():
    SUPER:UnknownStart().
    IF NOT clgMainDiagram THEN
      RETURN.

    DEFINE VARIABLE minDiagramID AS INTEGER     NO-UNDO.
    /* crear el diagrama general */
    minDiagramID = cobUtils:CreateDiagram(
        INPUT cinProjectComponentsPackageId,
        cchMainDiagramName,
        'Package'
        ).
    IF minDiagramID EQ ? THEN
      UNDO, THROW NEW AppError('No se puede crear el diagrama',292).
    cinCurrentObjectID = minDiagramID.
  END METHOD.
  
  METHOD PROTECTED OVERRIDE VOID FunctionalUnitStart():
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.
    DEFINE VARIABLE minDiagramID AS INTEGER     NO-UNDO.
    DEFINE VARIABLE mchName AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchPackName AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchType AS CHARACTER INITIAL 'nsInitialProgram'  NO-UNDO.

    ASSIGN
      mchPackName = cobParser:attributeValue("path")
      mchName = cobParser:AttributeValue("program")
      .

    minObjectID = cobUtils:FindPackage (
          input mchPackName,
          input cinProjectComponentsPackageId
          ).
    if  minObjectID eq ? or minObjectID eq 0 THEN DO: /* no est la unidad ? */
      cinCurrentObjectID = -1. /* "marcar" la ausencia */
      cobParser:DoLog(STRING(THIS-OBJECT),"No se encontr la unidad funcional " + mchName,'ERROR').
      RETURN.
    END.
    /* va al diagrama principal si es una UF raz */
    IF clgMainDiagram AND cobParser:attributeValue('parent') EQ ? THEN DO:
      DEFINE VARIABLE minFUOId AS INTEGER     NO-UNDO.
      minFUOid = cobUtils:FindObject (
        input "Package",
        input "nsFunctionalUnit",
        input mchPackName,
        ?
        ).                  
      if  minFUOid GT 0 THEN
        cobUtils:AddDiagramObject(
          INPUT cinCurrentObjectID,
          INPUT minFUOid,
          '',
          FALSE
          ).
    END.
    /* crear el diagrama para le unidad funcional */
    minDiagramID = cobUtils:CreateDiagram(
        INPUT minObjectID,
        INPUT mchPackName,
        "Component"
        ).
    IF minDiagramID EQ ? THEN
      UNDO, THROW NEW AppError('No se puede crear el diagrama',292).
    cchFUNAme = mchPackName.
    assign
      cinCurrentObjectID = minDiagramID
      cinCurrentPackageID = minObjectID
      mchType = 'nsInclude' WHEN (mchName EQ ? OR mchName EQ '')
      mchName = cobParser:AttributeValue('include') WHEN (mchName EQ ? OR mchName EQ '')
      .
    minObjectID = cobUtils:FindObject(
      input "Component",
      input mchType,
      input mchName,
      cinCurrentPackageID
      ).
    cobUtils:AddDiagramObject(
        INPUT minDiagramID,
        INPUT minObjectID,
        '',
        FALSE
        ).
  END METHOD.

  METHOD PROTECTED OVERRIDE VOID IncludeStart():
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.
    DEFINE VARIABLE mchFile AS CHARACTER   NO-UNDO.
          
    IF cinCurrentObjectID  EQ ? OR cinCurrentObjectID EQ 0 THEN
      RETURN.
    mchFile = cobParser:AttributeValue("file").
    IF mchFile MATCHES cchClasifMatch THEN
        minObjectID = cobUtils:FindObject (
                input "Component",
                input "nsClasifInclude",
                input mchFile,
                INPUT cinClasifComponentsPackageId                
                ).                  
    ELSE
        minObjectID = cobUtils:FindObject (
                input "Component",
                input "nsInclude",
                input mchFile,
                cinCurrentPackageID
                ).                  
    if     minObjectID eq ?
        or minObjectID eq 0
    THEN DO:
      cobParser:DoLog(STRING(THIS-OBJECT),'No se encuentra el archivo de inclusin ' + quoter(cobParser:attributeValue('file')),'ERROR').
      RETURN.
    END.
    cobUtils:AddDiagramObject(
        INPUT cinCurrentObjectID,
        INPUT minObjectID,
        '',
        FALSE
        ).
  END METHOD.

  METHOD PROTECTED OVERRIDE VOID ProgramStart():
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.
          
    IF cinCurrentObjectID EQ ? OR cinCurrentObjectID EQ 0 THEN
      RETURN.
    minObjectID = cobUtils:FindObject (
                input "Component",
                input "nsProgram",
                input cobParser:AttributeValue("file"),
                cinCurrentPackageID
                ).                  
    if     minObjectID eq ?
        or minObjectID eq 0
    THEN DO:
      cobParser:DoLog(STRING(THIS-OBJECT),'No se encuentra el archivo de programa ' + quoter(cobParser:attributeValue('file')),'ERROR').
      RETURN.
    END.
    cobUtils:AddDiagramObject(
        INPUT cinCurrentObjectID,
        INPUT minObjectID,
        '',
        FALSE
        ).
  END METHOD.
  
  METHOD PROTECTED OVERRIDE VOID CallStart():
    DEFINE VARIABLE mchCalledName AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.
        
    IF cinCurrentObjectID EQ ? OR cinCurrentObjectID EQ 0 THEN
      RETURN.
    mchCalledName = cobParser:AttributeValue('component').
    ASSIGN
      mchCalledName = cobParser:AttributeValue("fu") WHEN (mchCalledName EQ ?)
      .
    IF mchCalledName MATCHES  cchClasifMatch THEN DO:
      minObjectID = cobUtils:FindObject(
        input "Component",
        input "nsClasifInclude",
        input mchCalledName,
        cinClasifComponentsPackageId
        ).
      IF minObjectID EQ ? OR minObjectID EQ 0 THEN DO:
        minObjectID = cobUtils:FindObject(
          input "Component",
          input "nsClasifProgram",
          input mchCalledName,
          cinClasifComponentsPackageId
          ).
      END.
    END.
    ELSE IF mchCalledName MATCHES  cchLibMatch THEN DO:
      minObjectID = cobUtils:FindObject(
        input "Component",
        input "nsLibInclude",
        input mchCalledName,
        cinnsLibComponentsPackageId
        ).
      IF minObjectID EQ ? OR minObjectID EQ 0 THEN DO:
        minObjectID = cobUtils:FindObject(
          input "Component",
          input "nsLibProgram",
          input mchCalledName,
          cinnsLibComponentsPackageId
          ).
      END.
    END.
    ELSE DO:
      mchCalledName = cobParser:AttributeValue("fu").
      minObjectID = cobUtils:FindObject (
        input "Package",
        input "nsFunctionalUnit",
        input mchCalledName,
        ?
        ).                  
    END.
    /* es aceptable que falten unidades funcionales */
    if  minObjectID eq ? or minObjectID eq 0 THEN DO:
      cobParser:DoLog(STRING(THIS-OBJECT),'No se encuentra la unidad funcional ' + quoter(mchCalledName),'ERROR').
      RETURN.
    END.
    cobUtils:AddDiagramObject(
      INPUT cinCurrentObjectID,
      INPUT minObjectID,
      '',
      FALSE
      ).
  END METHOD.

  METHOD PROTECTED OVERRIDE VOID TableRefStart():
    DEFINE VARIABLE minObjectId AS INTEGER     NO-UNDO.
    DEFINE VARIABLE mchTableName AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchDBName AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE minDBID AS INTEGER     NO-UNDO.

    IF cinCurrentObjectID EQ ? OR cinCurrentObjectID EQ 0 THEN
      RETURN.
    mchTableName = SplitDBTable(cobParser:AttributeValue("table"),OUTPUT mchDBName).
    minDBID = cobUtils:FindPackage(
        mchDBName,
        cinDataModelPackageID
        ).
    minObjectID = cobUtils:FindObject (
        input "Class",
        input "nsTable",
        input mchTableName,
        INPUT minDBID
        ).
    /* si la tabla no exite, ver si existe la secuencia */
    IF minObjectId EQ ? OR minObjectID EQ 0 THEN
        minObjectID = cobUtils:FindObject(
            INPUT "Class",
            INPUT "nsSequence",
            INPUT mchTableName,
            INPUT minDBID
            ).

    if     minObjectID eq ?
        or minObjectID eq 0
    THEN DO:
      cobParser:DoLog(STRING(THIS-OBJECT),'No se encuentra la tabla ' + quoter(cobParser:AttributeValue("table")),'ERROR').
      RETURN.
    END.
    cobUtils:AddDiagramObject(
      INPUT cinCurrentObjectID,
      INPUT minObjectID,
      '',
      FALSE
      ).
  end METHOD.

  METHOD PROTECTED OVERRIDE VOID FunctionalUnitEnd():
    cinCurrentObjectID = ?.
    cinCurrentPackageID = ?.
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID UnknownEnd():
    IF clgMainDiagram THEN
      cinCurrentObjectID = ?.
  END METHOD.

  CONSTRUCTOR DiagramWorker(INPUT ipobUtils AS CLASS IUtils):
    SUPER(ipobUtils).
  END CONSTRUCTOR.
end CLASS.
