/*****************
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor original: T. Mercer-Hursh (thomas@cintegrity.com)
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Se encarga de crear los paquetes y componentes que representan las
unidades funcionales, procedimientos y archivos de inclusin.
- ENGLISH -
Creates packages and components to represent functional units,
procedure and include files.
*****************/
ROUTINE-LEVEL ON ERROR UNDO, THROW.

USING PROGRESS.Lang.AppError.
USING com.nomadesoft.ABLaUML.FUWorker.
USING com.nomadesoft.lib.*.
USING com.nomadesoft.ABLaUML.repository.*.

/**
Se encarga de crear todos los objetos y paquetes necesarios para luego poder procesar las relaciones
entre ellos.
**/
CLASS com.nomadesoft.ABLaUML.ObjectsWorker INHERITS FUWorker:

  define PROTECTED temp-table cttCodePackage no-undo
    field chType as character
    field chPackageName as character
    field inPackageID as integer
    field inObjectID as integer
    index ixByName as primary unique chType chPackageName
    .
  
  DEFINE PUBLIC PROPERTY clgFollowPackagePaths AS LOGICAL NO-UNDO
    GET.
    SET.

  METHOD PROTECTED VOID ParsePackage(INPUT ipchPackageFullName as character,output opchPackageParent as CHARACTER,output opchPackageBaseName as CHARACTER):

    define variable iinSlash as integer    no-undo.

    iinSlash = r-index(ipchPackageFullName,"/").
    if iinSlash eq 0
    then iinSlash = r-index(ipchPackageFullName,"~\").
    if iinSlash eq 0
    THEN
      assign
          opchPackageParent = ""
          opchPackageBaseName = ipchPackageFullName
          .
    else
      assign
          opchPackageParent = substring(ipchPackageFullName, 1, iinSlash - 1 )
          opchPackageBaseName = substring(ipchPackageFullName, iinSlash + 1 )
          .
  END METHOD.

  METHOD PROTECTED INTEGER FindPackageObject(INPUT ipinPackageId AS INTEGER):
    find cttCodePackage
        where cttCodePackage.inPackageID eq ipinPackageId 
        no-error.
    IF NOT AVAILABLE cttCodePackage THEN
      RETURN ?.
    RETURN cttCodePackage.inObjectID.
  END METHOD.

  METHOD PROTECTED INTEGER FindPackage(input ipchType as character,input ipchPackageName as character,input ipinParentPackageID as integer,input ipchStereotype as CHARACTER):
      define variable mchPackageParentName as character  no-undo.
      define variable mchPackageBaseName as character no-undo.
      define variable minLocalParentPackageID as integer no-undo.
      define variable minLocalPackageID as integer no-undo.
      define variable minLocalObjectID as integer no-undo.

      define variable iinLocalParentObjectID as integer no-undo.
      define variable ichNotes as character  no-undo.

      find cttCodePackage
          where cttCodePackage.chType eq ipchType 
          and   cttCodePackage.chPackage eq ipchPackageName 
          no-error.

      if not available( cttCodePackage ) then do:
        IF clgFollowPackagePaths THEN
          ParsePackage(ipchPackageName,output mchPackageParentName,output mchPackageBaseName).
        if mchPackageParentName ne "" THEN
          minLocalParentPackageID = FindPackage (ipchType,mchPackageParentName,ipinParentPackageID,ipchStereotype).
        ELSE
          minLocalParentPackageID = ipinParentPackageID.
        /* verificar que el paquete no exista ya, aunque est en otro lugar */
        minLocalPackageID = cobUtils:FindPackage(ipchPackageName,?).
        IF minLocalPackageID EQ 0 OR minLocalPackageID EQ ? THEN DO: /* Create package */
          minLocalPackageID = cobUtils:CreatePackage (ipchPackageName,minLocalParentPackageID).
                                                   /* Create corresponding object */
          minLocalObjectID = cobUtils:CreateObject("Package",ipchPackageName,minLocalPackageID,minLocalParentPackageID,ipchStereotype,"","").
        END.
        ELSE
          minLocalObjectID = cobUtils:FindObject("Package",ipchStereotype,ipchPackageName,minLocalPackageID).
                                                       /* Create Temp Table entry */
        create cttCodePackage.
        assign
            cttCodePackage.chType = ipchType
            cttCodePackage.chPackageName = ipchPackageName
            cttCodePackage.inPackageID = minLocalPackageID
            cttCodePackage.inObjectID = minLocalObjectID
            .
      end.  
      RETURN cttCodePackage.inPackageID.
  END METHOD.

  METHOD PROTECTED OVERRIDE VOID FunctionalUnitStart():
    DEFINE VARIABLE minFUProgramID AS INTEGER     NO-UNDO.
    DEFINE VARIABLE minTagID AS INTEGER     NO-UNDO.
    DEFINE VARIABLE mchProgramName AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchStereotype AS CHARACTER  INITIAL 'nsInitialProgram' NO-UNDO.

    mchProgramName = cobParser:AttributeValue("program").
    assign
        mchProgramName = cobParser:AttributeValue("include") WHEN (mchProgramName EQ ? OR mchProgramName EQ '')
        .

    /* obtener el paquete para esta unidad funcional, si no existe se crear */
    cinCurrentPackageId = FindPackage ("Functional Units",cobParser:AttributeValue('path'),cinProjectComponentsPackageId,"nsFunctionalUnit").
    minFUProgramID = FindPackageObject(cinCurrentPackageID).
    if cinCurrentPackageID eq ? OR minFUProgramID EQ ? then
      UNDO,throw NEW AppError("Failed to create program unit = " + cobParser:AttributeValue('path'),7 ).
    cinCurrentObjectID = minFUProgramID.
                                                 /* Create Program Unit */
    if cobParser:AttributeValue('program') EQ ? THEN
      mchStereotype = "nsInclude".   
    minFUProgramID = cobUtils:CreateObject (
        input "Component",
        input mchProgramName, /* ruta completa, en lugar de BaseName !!*/
        input cinCurrentPackageID,
        input 0,
        input mchStereotype,
        input '',
        input ''
        ).
    if minFuProgramID eq ?
    then
      UNDO, THROW NEW AppError("Failed to create program unit object = " + mchProgramName,8 ).
                                                   /* Set default HasDA */
    minTagID = cobUtils:SetObjectTag (
        input minFUProgramID,
        input "HasDA",
        input "false"
        ).             
    if minTagID eq ?
    then
      undo,throw NEW AppError("Failed to create program unit HasDA tag for " + mchProgramName,9 ).
  END METHOD.

  METHOD PROTECTED OVERRIDE VOID IncludeStart():
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.
    DEFINE VARIABLE mchFile AS CHARACTER   NO-UNDO.

    mchFile = cobParser:AttributeValue('file').
    /* si se trata de un archivo de clasif, se lo crea en el paquete correspondiente
    a clasif, aunque en realidad debera repetirse en cada lugar que se incluya */
    IF mchFile MATCHES  cchClasifMatch THEN DO:
        minObjectID = cobUtils:FindObject(
          input "Component",
          input "nsClasifInclude",
          input mchFile,
          INPUT cinClasifComponentsPackageId
          ).
        IF minObjectID EQ ? THEN
            minObjectID = cobUtils:CreateObject (
                input "Component",
                input mchFile,
                input cinClasifComponentsPackageId,
                input 0,
                input 'nsClasifInclude',
                input "",
                input ""
                ).
    END.
    ELSE
        minObjectID = cobUtils:CreateObject (
            input "Component",
            input mchFile, /* ruta completa en lugar de BaseName !! */
            input cinCurrentPackageID,
            input 0,
            input "nsInclude",
            input "",
            input ""
            ).          
    if minObjectID eq ?
    then
      undo, throw new AppError("CreatObject failed for include unit = " + cobParser:AttributeValue('file'),15 ).
    cinCurrentObjectID = minObjectID.
  END METHOD.

  METHOD PROTECTED OVERRIDE VOID ProgramStart():
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.

    minObjectID = cobUtils:CreateObject (
        input "Component",
        input cobParser:AttributeValue('file'), /* ruta completa en lugar de BaseName !! */
        input cinCurrentPackageID,
        input 0,
        input "nsProgram",
        input "",
        input ""
        ).          
    if minObjectID eq ?
    then
      undo, throw new AppError("CreatObject failed for include unit = " + cobParser:AttributeValue('file'),15 ).
    cinCurrentObjectID = minObjectID.
  END METHOD.

  METHOD PROTECTED OVERRIDE VOID CallStart():
    DEFINE VARIABLE mchCalledPackage AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.
    DEFINE VARIABLE mlgEsClasif AS LOGICAL     NO-UNDO.

    /* invocacin a otra unidad funcional, garantizar que la unidad funcional invocada exista */
    mchCalledPackage = cobParser:AttributeValue("fu").
    ASSIGN
      mchCalledPackage = cobParser:AttributeValue('component') WHEN (mchCalledPackage EQ ? OR mchCalledPackage EQ '')
      mlgEsClasif = mchCalledPackage MATCHES  cchClasifMatch
      .
    /* si se trata de una unidad funcional del paquete actual si no existe an va a crearse eventualmente */
    IF NOT mlgEsClasif AND NOT (mchCalledPackage MATCHES  cchLibMatch) THEN DO:
      cinCurrentObjectID = -1. /* el cierre igual va a ejecutarse, por lo cual agrego un valor pedorro para no romper la cadena */
      RETURN.
    END.
    minObjectID = cobUtils:FindObject(
      input "Component",
      input "nsFunctionalUnit",
      input mchCalledPackage,
      ?
      ).
    IF minObjectID EQ ? THEN DO:
      DEFINE VARIABLE mchEstereotipo AS CHARACTER INITIAL 'nsLibInclude'   NO-UNDO.
      mchCalledPackage = cobParser:AttributeValue('component').
      IF mlgEsClasif THEN DO:
        IF NUM-ENTRIES(mchCalledPackage,'.') GT 1 AND ENTRY(2,mchCalledPackage,'.') NE 'i' THEN
          mchEstereotipo = 'nsClasifProgram'.
        ELSE
          mchEstereotipo = 'nsClasifInclude'.
        minObjectID = cobUtils:CreateObject (
            input "Component",
            input mchCalledPackage,
            input cinClasifComponentsPackageId,
            input 0,
            input mchEstereotipo,
            input "",
            input ""
            ).
      END.
      ELSE DO:
        IF NUM-ENTRIES(mchCalledPackage,'.') GT 1 AND ENTRY(2,mchCalledPackage,'.') NE 'i' THEN
          mchEstereotipo = 'nsLibProgram'.
        minObjectID = cobUtils:CreateObject (
            input "Component",
            input mchCalledPackage,
            input cinnsLibComponentsPackageId,
            input 0,
            input mchEstereotipo,
            input "",
            input ""
            ).
      END.

      if minObjectID eq ?
      then
        UNDO, THROW NEW AppError("Failed to create program unit object = " + mchCalledPackage,8 ).
                                                     /* Set default HasDA */
      IF cobUtils:SetObjectTag (
          input minObjectID,
          input "HasDA",
          input "false"
          ) eq ?
      then
        undo,throw NEW AppError("Failed to create program unit HasDA tag for " + mchCalledPackage,9 ).
    END.
    cinCurrentObjectID = minObjectID.
  END METHOD.

  METHOD PROTECTED OVERRIDE VOID TableRefStart():
    DEFINE VARIABLE mchTableName AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchDBName AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE minObjectID AS INTEGER     NO-UNDO.
    DEFINE VARIABLE minDBID AS INTEGER     NO-UNDO.

    IF cobUtils:SetObjectTag (
        input cinCurrentObjectID,
        input "HasDA",
        input "true"
        ) eq ?
    then
      UNDO, THROW new AppError("PU HasDA tag failed",26 ).

    /* garantizar que la tabla est definida */
    mchTableName = SplitDBTable(cobParser:AttributeValue("table"),OUTPUT mchDBName).
      .
    minDBID = cobUtils:FindPackage(mchDBName,cinDataModelPackageID).
    IF minDBID EQ ? OR minDBID EQ 0 THEN DO:
      minDBID = cobUtils:CreatePackage (
          input mchDBName,
          input cinDataModelPackageID
          ). 
      if minDBID eq ? or minDBID eq 0 then
        undo, THROW NEW AppError("Failure to Create Package for " + mchDBName,1).
                                                   /* Create corresponding object */
      cobUtils:CreateObject (
        input "Package",
        input mchDBName,
        INPUT minDBID,
        input cinDataModelPackageID,
        input "nsDatabase",
        input "",
        input ""
        ).
    END.
    minObjectID = cobUtils:FindObject (
        input "Class",
        input "nsTable",
        input mchTableName,
        minDBID
        ).
    if     minObjectID eq ?
        or minObjectID eq 0
    THEN DO:
      /* primero tengo que verificar si existe una secuencia con ese nombre.
      Recin entonces creo la tabla faltante. */
      minObjectID = cobUtils:FindObject(
          INPUT "Class",
          INPUT "nsSequence",
          INPUT mchTableName,
          INPUT minDBID
          ).
      IF minObjectId EQ ? OR minObjectId EQ 0 THEN
          minObjectID = cobUtils:CreateObject (
              input "Class",
              input mchTableName,
              input minDBID,
              input 0,
              input "nsTable",
              input "",
              input ""
              ).
      if     minObjectID eq ?
          or minObjectID eq 0 THEN
        undo, throw new AppError("CreateObject failed for " + mchTableName + " in tableref ",242 ).
    END.
  END METHOD.

  METHOD PROTECTED OVERRIDE VOID FieldRefStart():
    IF cobUtils:SetObjectTag (
        input cinCurrentObjectID,
        input "HasDA",
        input "true"
        ) eq ?
    then
      UNDO, THROW new AppError("PU HasDA tag failed",26 ).
  END METHOD.

  METHOD PROTECTED OVERRIDE VOID FunctionalUnitEnd():
    cinCurrentObjectID = ?.
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID IncludeEnd():
    cinCurrentObjectID = ?.
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID ProgramEnd():
    cinCurrentObjectID = ?.
  END METHOD.
  METHOD PROTECTED OVERRIDE VOID CallEnd():
    cinCurrentObjectID = ?.
  END METHOD.

  CONSTRUCTOR ObjectsWorker(INPUT ipobUtils AS CLASS IUtils):
    SUPER(ipobUtils).
  END CONSTRUCTOR.
END CLASS.
