/*****************
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Clase base para generar archivos XML a ser utilizados para la
construccin de modelos EA.
- ENGLISH -
Base class for XML generation to be used in EA model building.
*****************/

&GLOBAL-DEFINE END .
ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS com.nomadesoft.lib.XMLwriter:

  /* archivo de salida del xml */
  DEFINE PRIVATE STREAM cstOutput.
  DEFINE PRIVATE VARIABLE clgStreamOpen AS LOGICAL NO-UNDO.

  /* si la variable est en TRUE entonces la salida no se enva al archivo, sino que se almacena en "buffer" */
  DEFINE PRIVATE VARIABLE clgBuffering AS LOGICAL     NO-UNDO.
  DEFINE PRIVATE VARIABLE cchBuffer AS CHARACTER   NO-UNDO.

  /* conserva los nombres de los marcadores anidados, para poder cerrarlos */
  DEFINE PRIVATE VARIABLE cchTagHistory AS CHARACTER  INITIAL ? NO-UNDO.

  /* determina si se generan saltos de lnea y tabulaciones o simplemente se genera el chorro de xml */
  DEFINE PUBLIC PROPERTY clgFormatOutput AS LOGICAL NO-UNDO
    GET.
    SET.

  /* profundidad del marcador actual */
  DEFINE PROTECTED PROPERTY cinTagLevel AS INTEGER NO-UNDO
    GET.
    SET.


  DEFINE PUBLIC PROPERTY cchCharset AS CHARACTER INITIAL 'ISO8859-1' NO-UNDO
    GET.
    PROTECTED SET.

  /* nombre del marcador actualmente abierto, mantiene el historial, recupera el anterior cuando se le asigna ? */
  DEFINE PROTECTED PROPERTY cchCurrentTag AS CHARACTER INITIAL ? NO-UNDO
    GET.
    PRIVATE SET(INPUT ipchTagName AS CHARACTER):
      DEFINE VARIABLE minEntries AS INTEGER     NO-UNDO.
      IF ipchTagName EQ ? THEN DO:
        IF cchTagHistory EQ ? THEN
          cchCurrentTag = ?.
        ELSE DO:
          ASSIGN
            minEntries = NUM-ENTRIES(cchTagHistory)
            cchCurrentTag = ENTRY(minEntries,cchTagHistory)
            ENTRY(minEntries,cchTagHistory) = ''
          {&END}
          IF minEntries > 1 THEN
            SUBSTRING(cchTagHistory,LENGTH(cchTagHistory),1) = ''.
          ELSE
            cchTagHistory = ?.
        END.
      END.
      ELSE ASSIGN
        cchTagHistory = '' WHEN (cchTagHistory EQ ? AND cchCurrentTag NE ?)
        cchTagHistory = cchTagHistory + ',' WHEN (cchTagHistory NE '' AND cchTagHistory NE ? AND cchCurrentTag NE ?)
        cchTagHistory = cchTagHistory + cchCurrentTag WHEN (cchCurrentTag NE ?)
        cchCurrentTag = ipchTagName
      {&END}
    END SET.

  DEFINE PROTECTED PROPERTY cchLineEnd AS CHARACTER NO-UNDO
    GET():
      IF clgFormatOutput THEN
        RETURN '~n'.
      RETURN ''.
    END GET.

  METHOD PRIVATE VOID OutLine(INPUT ipchxml AS CHARACTER):
    IF NOT clgBuffering THEN
      PUT STREAM cstOutput UNFORMATTED ipchxml cchLineEnd.
    ELSE
      cchBuffer = cchBuffer + ipchxml + cchLineEnd.
  END METHOD.

  /* genera las tabulaciones adecuadas al nivel actual */
  METHOD PRIVATE CHARACTER makeOutputLevel():
    IF NOT clgFormatOutput THEN
      RETURN ''.
    RETURN FILL('~t',cinTagLevel).
  END METHOD.

  /* recibe una lista de pares nombre,valor y devuelve una cadena en la forma de atributos xml */
  METHOD PRIVATE CHARACTER makeTagAttributes(INPUT ipchAtts AS CHARACTER):
    DEFINE VARIABLE minIndex AS INTEGER     NO-UNDO.
    DEFINE VARIABLE mchAtts AS CHARACTER   NO-UNDO.

    IF ipchAtts EQ ? OR ipchAtts EQ '' THEN
      RETURN ''.
    DO minIndex = 1 TO NUM-ENTRIES(ipchAtts) BY 2:
      mchAtts = mchAtts + ' ' + ENTRY(minIndex,ipchAtts) + '="' + REPLACE(ENTRY(minIndex + 1,ipchAtts),CHR(4),',') + '"'.
    END.
    RETURN mchAtts.
  END METHOD.

  /* escapa una cadena para poder formar la lista de atributos, se va a "desescapar" al armar los atributros */
  METHOD PROTECTED CHARACTER Escapar(INPUT ipchData AS CHARACTER):
    /* los atributos no pueden contener '<' ni '>', ni tampoco comillas */
    ASSIGN
      ipchData = REPLACE(ipchData,'&','&amp;')
      ipchData = REPLACE(ipchData,'<','&lt;')
      ipchData = REPLACE(ipchData,'>','&gt;')
      ipchData = REPLACE(ipchData,'"',"&quot;")
      .
    IF cchCharset NE SESSION:CPINTERNAL THEN
      ipchData = CODEPAGE-CONVERT(ipchData,cchCharset).
    RETURN REPLACE(ipchData,',',CHR(4)).
  END METHOD.

  /* genera el xml para el marcador con el contenido y atributos especificados. Si el contenido es ? se genera un marcador simple. */
  METHOD PROTECTED VOID OutputTag(INPUT ipchTagName AS CHARACTER, INPUT ipchTagAttributes AS CHARACTER, INPUT ipchTagContent AS CHARACTER):
    DEFINE VARIABLE mchXML AS CHARACTER   NO-UNDO.
    DEFINE VARIABLE mchOLevel AS CHARACTER   NO-UNDO.

    mchXML = makeOutputLevel() + '<' + ipchTagName + makeTagAttributes(ipchTagAttributes).

    IF ipchTagContent EQ ? THEN
      mchXML = mchXML + '/>'.
    ELSE IF clgFormatOutput THEN DO:
      cinTagLevel = cinTagLevel + 1.
      mchOLevel = makeOutputLevel().
      cinTagLevel = cinTagLevel - 1.
      mchXML = mchXML + '><![CDATA[' + cchLineEnd + mchOLevel + REPLACE(ipchTagContent,cchLineEnd,cchLineEnd + mchOLevel) + cchLineEnd + makeOutputLevel() + ']]></' + ipchTagName + '>'.
    END.
    ELSE
      mchXML = mchXML + '><![CDATA[' + ipchTagContent + ']]></' + ipchTagName + '>'.
    OutLine(mchXML).
  END METHOD.

  METHOD PROTECTED VOID StartTag(INPUT ipchTagNAme AS CHARACTER, INPUT ipchTagAttributes AS CHARACTER):
    OutLine(makeOutputLevel() + '<' + ipchTagName + makeTagAttributes(ipchTagAttributes) + '>').
    cinTagLevel = cinTagLevel + 1.
    cchCurrentTag = ipchTagName.
  END METHOD.

  METHOD PROTECTED VOID EndTag():
    IF cchCurrentTag EQ ? THEN
      RETURN.
    cinTagLevel = cinTagLevel - 1.
    OutLine(makeOutputLevel() + '</' + cchCurrentTag + '>').
    cchCurrentTag = ?.
  END METHOD.

  /* genera la cabecera xml y abre la raz */
  METHOD PROTECTED VOID StartXML(INPUT ipchRootName AS CHARACTER,INPUT ipchAttributes AS CHARACTER, INPUT ipchDestino AS CHARACTER):
    IF clgStreamOpen THEN
      FinishXML().
    clgBuffering = clgBuffering OR ipchDestino EQ ?.
    IF NOT clgBuffering THEN DO:
      OUTPUT STREAM cstOutput TO VALUE(ipchDestino).
      clgStreamOpen = TRUE.
    END.
    OutLine('<?xml version="1.0" encoding="' + cchCharset + '"?>' + cchLineEnd).
    StartTag(ipchRootName,ipchAttributes).
  END METHOD.

  /* cierra todos los marcadores abiertos hasta completar el xml */
  METHOD PROTECTED VOID FinishXML():
    DO WHILE cinTagLevel GT 0:
      EndTag().
    END.
    IF clgStreamOpen THEN
      OUTPUT STREAM cstOutput CLOSE.
    clgStreamOpen = FALSE.
  END METHOD.

  METHOD PUBLIC VOID writeXML(INPUT ipchDestino AS CHARACTER):
    StartXML('raiz',?,ipchDestino).
    OutputTag('error','quien,' + STRING(THIS-OBJECT),'Se ha invocado el mtodo writeXML de la clase base').
    FinishXML().
  END METHOD.

  DESTRUCTOR XMLwriter():
    FinishXML().
  END DESTRUCTOR.
END CLASS.
