/*****************
Propiedad intelectual de NmadeSoft SRL (info@nomadesoft.com.ar)
---
autor: D. Abdala (dabdala@nomadesoft.com.ar)
ltima modificacin: 2010-06-01
---
Genera un archivo xml con la informacin de la estructura de una
o ms bases de datos.
- ENGLISH -
Generates an xml file with database structure information, file
can contain the structure of one or more databases.
*****************/

ROUTINE-LEVEL ON ERROR UNDO, THROW.
USING com.nomadesoft.xml.DDictwriter.
USING com.nomadesoft.lib.Logger.

/* ***************************  Definitions  ******************************** */
DEFINE VARIABLE lobLogger AS CLASS Logger NO-UNDO.
DEFINE VARIABLE lobWrite AS CLASS DDictwriter NO-UNDO.
DEFINE VARIABLE llgBonito AS LOGICAL LABEL 'Agregar otra base de datos' NO-UNDO.
DEFINE VARIABLE lchArchivo AS CHARACTER  FORMAT "x(160)" VIEW-AS FILL-IN SIZE-CHARS 40 BY 4 LABEL 'Destino' NO-UNDO.
DEFINE VARIABLE linIndex AS INTEGER   INITIAL 1  NO-UNDO.
DEFINE FRAME a WITH 1 COLUMNS.

/* *************************  Initialization ******************************** */
/* para la generacin utilizo el alias DictSource, por lo cual me garantizo de
que no est en uso */
DISCONNECT VALUE('DictSource').
DELETE ALIAS DictSource.

/* para generar necesito al menos una base de datos */
RUN connectDB('db1','sports2000','chonik',2561).
if not connected( "db1" ) then
  RETURN.

CREATE ALIAS DictSource FOR DATABASE 'db1'.
DEFINE STREAM stXML.
DEFINE STREAM stIn.
DEFINE VARIABLE lchLine AS CHARACTER   NO-UNDO.
DEFINE VARIABLE linPrevPos AS INTEGER  EXTENT 2   NO-UNDO.

SESSION:ERROR-STACK-TRACE = TRUE.

UPDATE lchArchivo WITH FRAME a. /* nombre del archivo xml */
IF lchArchivo NE ? AND lchArchivo NE '' THEN DO ON ERROR UNDO, THROW:
  lchArchivo = ENTRY(1,lchArchivo,'.').
  lobLogger = NEW Logger('nsDictXML.log').
  lobLogger:OutLog('Iniciando generacin de XML para DD',lobLogger:getLevel('MESSAGE')).
  /* la forma ms simple es generar un archivo para cada base de datos y luego
  condensarlos en un nico archivo */
  DO WHILE CONNECTED('db' + STRING(linIndex)) ON ERROR UNDO, THROW:
    lobWrite = NEW DDictwriter().
    lobWrite:cobLogger = lobLogger.
    lobWrite:clgFormatOutput = TRUE.
    lobLogger:OutLog('procesando base de datos N ' + STRING(linIndex),lobLogger:getLevel('MESSAGE')).
    lobWrite:writeXML(lchArchivo + STRING(linIndex) + '.xml').
    DISCONNECT VALUE('DictSource').
    llgBonito = FALSE.
    UPDATE llgBonito WITH FRAME b.
    IF llgBonito THEN DO:
      linIndex = linIndex + 1.
      /* conectar la nueva base de datos */
      DELETE ALIAS DictSource.
      RUN connectDB('db' + STRING(linIndex),'','',0).
      IF CONNECTED('db' + STRING(linIndex)) THEN
        CREATE ALIAS DictSource FOR DATABASE VALUE('db' + STRING(linIndex)).
    END.
    FINALLY:
      DELETE OBJECT lobWrite.
    END FINALLY.
  END.
  lobLogger:OutLog('generacin finalizada',lobLogger:getLevel('MESSAGE')).
  lobLogger:OutLog('unificando arvhivos',lobLogger:getLevel('MESSAGE')).
  
  /* condensar los archivos generados en un nico archivo */
  OUTPUT STREAM stXML TO VALUE(lchArchivo + '.xml').
  INPUT STREAM stIn FROM VALUE(lchArchivo + STRING(linIndex) + '.xml').
  DO WHILE TRUE ON ENDKEY UNDO, LEAVE:
    IMPORT STREAM stIn UNFORMATTED lchLine.
    ASSIGN
      linPrevPos[2] = linPrevPos[1]
      linPrevPos[1] = SEEK(stXML)
      .
    PUT STREAM stXML UNFORMATTED lchLine + '~n'.
  END.
  INPUT STREAM stIn CLOSE.
  DO WHILE linIndex GT 1:
    linIndex = linIndex - 1.
    SEEK STREAM stXML TO linPrevPos[1].
    INPUT STREAM stIn FROM VALUE(lchArchivo + STRING(linIndex) + '.xml').
    /* saltear cabecera y raiz */
    IMPORT STREAM stIn UNFORMATTED lchLine.
    IMPORT STREAM stIn UNFORMATTED lchLine.
    IMPORT STREAM stIn UNFORMATTED lchLine.
    DO WHILE TRUE ON ENDKEY UNDO, LEAVE:
      IMPORT STREAM stIn UNFORMATTED lchLine.
      ASSIGN
        linPrevPos[2] = linPrevPos[1]
        linPrevPos[1] = SEEK(stXML)
        .
      PUT STREAM stXML UNFORMATTED lchLine + '~n'.
    END.
    INPUT STREAM stIn CLOSE.
  END.
  OUTPUT STREAM stXML CLOSE.
  FINALLY:
    DELETE OBJECT lobLogger.
  END FINALLY.
END.
ELSE
  DISCONNECT VALUE('DictSource').

FINALLY:
  DELETE ALIAS DictSource.
END FINALLY.
